/*
        Display DECnet MOP SYSTEMID messages 

	These are broadcast to by all DECnet nodes on a LAN to AB-00-00-02-00 
	with protocol type 60-02 and contain varying amounts of info.
	(Usually the LAN interface type and hardware address and a few other
		bits and bobs).

	This program uses the famous undocumented NETACP QIO interface to
	attempt to translate DECnet addresses to names. Apart from that no other
	undocumented interfaces are used. Privs are required to run this.

	This program has been used in the following environments

		VAXC V3.2 VMS VAX V5.5-2. 
		DECC V5.0 VMS AXP V6.2
		DECC V6.5 VMS Alpha V7.3-2

	It may work on other versions.

	Any changes or bugs contact me at miller@encompasserve.org

 COPYRIGHT NOTICE

 This software is COPYRIGHT  1996, 2005, Ian Miller. 

 Released under licence described in aaareadme.txt 

 DISCLAIMER

 This software is provided "AS IS". The author makes no representations or
 warranties with respect to the software and specifically disclaim any implied
 warranties of merchantability or fitness for any particular purpose.

History	
=======

30-nov-1996 I.Miller  V1.0      Created - works for MOP V3 on VAX only.
10-oct-1996 I.Miller  V1.0-001  Add interpretation of info type 204 based on 
                                infomation from Galen Tackett.
                                Tell DECC not to pad structures - now this
                                program works on AXPVMS too!
11-OCT-1996 I.Miller  V1.0-002	Add XQA0 etc to ethernet device list.
				WR90 to comms device name list.
24-OCT-1996 I.Miller  V1.0-003	Display time in messages to terminal.
				If node already known don't exit just loop
				to read another message.
19-OCT-1996 I.Miller  V1.0-004  Add some more Alpha CPU ID (from Galen Tackett).
05-DEC-1996 I.Miller  V1.0-005	Add display of info types 205,206,207 based on
				info from Galen Tackett.
				Fix silly bug in code that prevents display
				of nodes more than once.
                                Split original SYSID.C into seperate files.
                                Incorporated better version of comms device
                                type table from Galen Tackett.
                                Add manufacturer table.
06-DEC-1996 I.Miller  V1.0-006	Display system up time (info type 209).
                                Tidy code a little.
                                Add more entries to mtable based on info
                                from Galen Tackett.
04-Feb-1997 I.Miller  V1.0-007  More tidying. some routines moved to another
                                file. Seems to work with MOP V4 SYSID too.
11-Feb-1997 I.Miller  V1.0-008	If Termial Server ROM Version not printable
				chars then output as three decimal numbers.
				(for DECserver 90L+).
04-Jul-2005 I.Miller  V1.0-009	Tidy code for newer compiler
*/

#define VERSION "V1.0-009"
#if __DECC
#pragma module SYSID VERSION
#else
#module SYSID VERSION 
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <descrip.h>

#include "ethernet.h"
#include "sysid.h"
#include "listen_subs.h"

static void display_comms_type(FILE *fpOut,unsigned char id);
static void display_data_link(FILE *fpOut,unsigned char id);
static void display_CPU_id(FILE *fpOut,unsigned char id);
static void display_software_id(FILE *fpOut,unsigned char *id,unsigned short len);
static void display_functions(FILE *fpOut,unsigned char *fns);
static void display_uptime(FILE *fpOut,unsigned char *info,unsigned short len);
static void  display_HW_model(FILE *fpOut,unsigned char *info,unsigned char info_len);

void 
format_sysid_msg(FILE *fpOut,int tty_output,FRAME_HEADER_ETH *hdr,
        unsigned char *msg, unsigned short len)
{
        unsigned int n;
        unsigned char *msgptr, *info;
        SYSID_HDR *msghdr;
        SYSID_INFO_HDR *infohdr;
	unsigned short dnet;

        msghdr = (SYSID_HDR *)msg;
	fprintf(fpOut,"SYSID from ");
        display_addr(fpOut,hdr->source);
	if (!tty_output)
	{
		printf("%s SYSID from ",curtime());
		display_addr(stdout,hdr->source);
	}
	if ((hdr->source[0] == 0xAA) && (hdr->source[1] == 0)
	&&  (hdr->source[2] == 0x04) && (hdr->source[3] == 0))
	{
		dnet = hdr->source[4] + hdr->source[5]*256;
		fprintf(fpOut," (%d.%d) %s",dnet/1024,dnet%1024,
			get_nodename(dnet));
		if (!tty_output)
		{
			fprintf(stdout," (%d.%d) %s",dnet/1024,dnet%1024,
				get_nodename(dnet));
		}
	}
	
	fputc('\n',fpOut);
	if (!tty_output)
	        fputc('\n',stdout);

        n = len - sizeof(SYSID_HDR);    /* length of other info data    */
        msgptr = &msg[sizeof(SYSID_HDR)];/* addr of first info           */
        while (n > 0)
        {
                infohdr = (SYSID_INFO_HDR *)msgptr;
		info = msgptr + sizeof(SYSID_INFO_HDR);
		if (infohdr->itype == 0)
			break;
		switch(infohdr->itype)
		{
		case 1:
			fprintf(fpOut,"  Maintainence version (%d,%d,%d)\n",
				info[0],info[1],info[2]);
			break;
		case 2:
			display_functions(fpOut,info);
			break;
		case 3:
			fprintf(fpOut,"  MOP Console user = ");
			display_addr(fpOut,info);
			fputc('\n',fpOut);
			break;
		case 4:
			fprintf(fpOut,"  Reservation timer = %d seconds\n",
				info[1]*256 + info[0]);
			break;
		case 5:
			fprintf(fpOut,"  Console cmd buffer size = %d\n",
				info[1]*256 + info[0]);
			break;
		case 6:
			fprintf(fpOut,"  Console rsponse buffer size = %d\n",
				info[1]*256 + info[0]);
			break;
		case 7:
			fprintf(fpOut,"  Hardware address = ");
			display_addr(fpOut,info);
                        display_man(fpOut,info);
			fputc('\n',fpOut);
			break;
		case 8:	/* reserved in V4 */
			fprintf(fpOut,"  System time %02d-%02d-%02d%02d %02d:%02d:%02d.%02d %02d:%02d\n",

				info[3],info[2],info[0],info[1],
				info[4],info[5],info[6],info[7],
				info[8],info[9]);
			break;
		case 9:	/* V4 */
			fprintf(fpOut,"  Node ID %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
			break;				
		case 10: /* V4 */
			fprintf(fpOut,"  System time %02d-%02d-%02d%02d %02d:%02d:%02d.%02d %02d:%02d\n",

				info[3],info[2],info[0],info[1],
				info[4],info[5],info[6],info[7],
				info[8],info[9]);
			break;
		case 11:	/* V4 */
			fprintf(fpOut,"  Node Name Part 1 %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
			break;				
		case 12:	/* V4 */
			fprintf(fpOut,"  Node Name Part 2 %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
			break;				
		case 13:	/* V4 */
			fprintf(fpOut,"  FDDI Station ID = ");
			display_bytes(fpOut,info,infohdr->ilen);
			fputc('\n',fpOut);
			break;
		case 100:
			display_comms_type(fpOut,info[0]);
			break;
                case 102:
			if (*info < ' ')
			{
				if (infohdr->ilen == 3)
				{
					fprintf(fpOut,
						"  Terminal Server ROM Version = %d.%d.%d\n",
						info[0],info[1],info[2]);
				}
				else
				{
					fprintf(fpOut,
						"  Terminal Server ROM Version =");
					display_bytes(fpOut,info,infohdr->ilen);
					fputc('\n',fpOut);
				}
			}
			else
			{
				fprintf(fpOut,
				    "  Terminal Server ROM Version = %*.*s\n",
                                    infohdr->ilen,infohdr->ilen,info);
			}
                        break;
                case 103:
                        fprintf(fpOut,"  Terminal Server Version = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
                case 104:
                        fprintf(fpOut,"  Terminal Server Number = %d\n",
                                info[0] + info[1]*256);
                        break;
                case 105:
                        fprintf(fpOut,"  Terminal Server Name = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
                case 106:
                        fprintf(fpOut,"  Terminal Server ID = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
		case 200:
			display_software_id(fpOut,info,infohdr->ilen);
			break;
                case 201:
                        fprintf(fpOut,"  Operating System ID = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
                case 202:
                        fprintf(fpOut,"  Operating System Version = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
                case 203:
                        fprintf(fpOut,"  Nodename = %*.*s\n",
                                infohdr->ilen,infohdr->ilen,info);
                        break;
                case 204:
                        display_HW_model(fpOut,info,infohdr->ilen);
                        break;
		case 205:
			fprintf(fpOut,"  Driver Version = ");
			display_lwords(fpOut,info,infohdr->ilen);
			fputc('\n',fpOut);
			break;
		case 206:
			fprintf(fpOut,"  LAN Version = ");
			display_lwords(fpOut,info,infohdr->ilen);
			fputc('\n',fpOut);
			break;
		case 207:
			fprintf(fpOut,"  LAN Device Firmware Version = ");
			display_lwords(fpOut,info,infohdr->ilen);
			fputc('\n',fpOut);
			break;
                case 209:
                        fprintf(fpOut,"  Software ID related type %d len %d\n",
					infohdr->itype,infohdr->ilen);
			display_bytes(fpOut,info,infohdr->ilen);
			display_uptime(fpOut,info,infohdr->ilen);
                        break;
		case 300:
			display_CPU_id(fpOut,info[0]);
			break;
		case 400:
			display_data_link(fpOut,info[0]);
			break;
		case 401:
			fprintf(fpOut,"  Data link buffer size %d\n",
				info[1]*256 + info[0]);
			break;
		default:
			if ((infohdr->itype > 100) && (infohdr->itype < 200))
			{
				fprintf(fpOut,"  Comms. Dev. related type %d len %d\n",
					infohdr->itype,infohdr->ilen);
			}
			else
			if ((infohdr->itype > 200) && (infohdr->itype < 300))
			{
				fprintf(fpOut,"  Software ID related type %d len %d\n",
					infohdr->itype,infohdr->ilen);
			}
			else
			if ((infohdr->itype > 300) && (infohdr->itype < 400))
			{
				fprintf(fpOut,"  CPU ID related type %d len %d\n",
					infohdr->itype,infohdr->ilen);
			}
			else
			if ((infohdr->itype > 401) && (infohdr->itype < 500))
			{
				fprintf(fpOut,"  Data link related type %d len %d\n",
					infohdr->itype,infohdr->ilen);
			}
			else
			{
	                	fprintf(fpOut,"  itype = %d, ilen = %d\n",
		                        infohdr->itype,infohdr->ilen);
			}
			display_bytes(fpOut,info,infohdr->ilen);
		}
                n -= sizeof(SYSID_INFO_HDR) + infohdr->ilen;
                msgptr += sizeof(SYSID_INFO_HDR) + infohdr->ilen;
        }
}

static void
display_functions(FILE *fpOut,unsigned char *info)
{
        unsigned short *fns;

        fns = (unsigned short *)info;

	fprintf(fpOut,"  Functions (0x%x)",*fns);
	if (*fns & 1)
		fprintf(fpOut," LOOP");
	if (*fns & 2)
		fprintf(fpOut," DUMP");
	if (*fns & 4)
		fprintf(fpOut," PRIMARY-LDR");
	if (*fns & 8)
		fprintf(fpOut," MULTI-LDR");
	if (*fns & 16)
		fprintf(fpOut," BOOT");
	if (*fns & 32)
		fprintf(fpOut," CONSOLE");
	if (*fns & 64)
		fprintf(fpOut," COUNTERS");
	if (*fns & 128)
		fprintf(fpOut," CONSOLE-RESERVED");
	fputc('\n',fpOut);
}

static void
display_software_id(FILE *fpOut,unsigned char *info,unsigned short len)
{
	char *id;

	id = (char *)info;

	fprintf(fpOut,"  Software ID = ");
	switch(*id)
	{
	case 0:
		fprintf(fpOut,"None (len = %d)\n",len);
		break;
	case -1:
		fprintf(fpOut,"Standard OS\n");
		break;
	case -2:
		fprintf(fpOut,"Maintainace system\n");
		break;
	default:
		if (*id > 0)
		{
			fprintf(fpOut,"%*.*s\n",*id,*id,id+1);
		}
		else
		{
			fprintf(fpOut,"%d\n",*id);
		}
	}
}

static void
display_CPU_id(FILE *fpOut,unsigned char id)
{
	fprintf(fpOut,"  CPU ID (%d) ",id);
	switch(id)
	{
	case 1:
		fputs("PDP-11\n",fpOut);
		break;
	case 2:
		fputs("Communication Server\n",fpOut);
		break;
	case 3:
		fputs("Professional\n",fpOut);
		break;
	case 9:
		fputs("Intel PC\n",fpOut);
		break;
	default:
		fputc('\n',fpOut);
	}
}

static void
display_data_link(FILE *fpOut,unsigned char id)
{
	fprintf(fpOut,"  Data link type (%d) ",id);
	switch(id)
	{
	case 1:
		fputs("Ethernet\n",fpOut);
		break;
	case 2:
		fputs("DDCMP\n",fpOut);
		break;
	case 3:
		fputs("LAPB\n",fpOut);
		break;
	case 4:
		fputs("HDLC\n",fpOut);
		break;	
	case 5:
		fputs("FDDI\n",fpOut);
		break;
	case 6:
		fputs("Token-passing Ring (IEEE 802.5)\n",fpOut);
		break;
	case 11:
		fputs("Token-passing Bus (IEEE 802.4)\n",fpOut);
		break;
	case 12:
		fputs("Z-LAN 4000: Zenith 4 MB/s broadband CSMA/CD LAN\n",fpOut);
		break;
	default:
		fputc('\n',fpOut);
	}
}

static void
display_comms_type(FILE *fpOut,unsigned char id)
{
typedef enum {sdc_undef,sdc_obsolete,sdc_lan,sdc_serial,sdc_other} SOFD_DEVCLASS;
typedef enum {sls_undef, sls_serial, sls_csmacd, sls_token, sls_fddi, sls_zlan,sls_unknown} SOFD_LAN_DLTYPE;

typedef struct _sofdtab_entry {
    const char *ste_mop_name;
    const char *ste_desc;
    SOFD_DEVCLASS ste_devclass;
    SOFD_LAN_DLTYPE ste_dltype;
} SOFDTAB_ENTRY;

static const SOFDTAB_ENTRY sofdtab[256] = {
    /* 0  */ { "DP", "DP11-DA", sdc_obsolete , 0 },
    /* 1  */ { "UNA", "DEUNA Unibus CSMA/CD", sdc_lan , sls_csmacd },
    /* 2  */ { "DU", "DU11-DA sync line" , sdc_serial , 0 },
    /* 3  */ { "CNA","DECNA Professional 3xx CSMA/CD" , sdc_lan , sls_csmacd},
    /* 4  */ { "DL","DL11-C/-E/-WA sync line) " , sdc_serial , 0 },
    /* 5  */ { "QNA","DEQNA Q-bus CSMA/CD" , sdc_lan , sls_csmacd },
    /* 6  */ { "DQ","DQ11-DA" , sdc_obsolete , 0 },
    /* 7  */ { "CI","CI Computer Interconnect" , sdc_other , 0 },
    /* 8  */ { "DA","DA11-B/-AL Unibus link" , sdc_other , 0 },
    /* 9  */ { "PCL","PCL11-B multiple CPU link" , sdc_other , 0 },
    /* 10 */ { "DUP","DUP11-DA sync line" , sdc_serial , 0 },
    /* 11 */ { "LUA","DELUA Unibus Lance CSMA/CD" , sdc_lan , sls_csmacd },
    /* 12 */ { "DMC","DMC11-DA/AR, -FA/AR, -MA/AL, -MD/AL interprocessor link)", sdc_serial},
    /* 13 */ { "LNA","DEMSA MicroServer Lance CSMA/CD", sdc_lan , sls_csmacd},
    /* 14 */ { "DN","DN11-BA/-AA automatic calling unit" , sdc_other , 0 },
    /* 15 */ { 0 },
    /* 16 */ { "DLV","DLV11-E/-F/-J, MXV11-A/-B async line" , sdc_serial , 0},
    /* 17 */ { "LCS","DSRVA DECserver 100 Lance CSMA/CD", sdc_lan ,sls_csmacd },
    /* 18 */ { "DMP","DMP11 multipoint interprocessor link" , sdc_other , 0},
    /* 19 */ { "AMB","AMBER", sdc_obsolete , 0 },
    /* 20 */ { "DTE","DTE20 PDP-11 to KL10 interface", sdc_other , 0 },
    /* 21 */ { "DBT","DEBET-AA LANbridge 100 CSMA/CD" , sdc_lan , sls_csmacd},
    /* 22 */ { "DV","DV11-AA/BA sync line multiplexer", sdc_serial , 0 },
    /* 23 */ { "BNA","DEBNA or obsolete DEBNT BI CSMA/CD ", sdc_lan ,sls_csmacd },
    /* 24 */ { "DZ","DZ11-A/-B/-C/-D async line multiplexer)" , sdc_serial ,0 },
    /* 25 */ { "LPC","PC500 VAXmate integral Lance CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 26 */ { "DSV","DSV11 Q-bus sync controller" , sdc_serial , 0 },
    /* 27 */ { "CEC","3Com 3C501 IBM-PC CSMA/CD" , sdc_lan , sls_csmacd },
    /* 28 */ { "KDP","KMC11/DUP11-DA sync line multiplexer" , sdc_serial , 0},
    /* 29 */ { "IEC","Micom/Interlan 5010 IBM-PC CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 30 */ { "KDZ","KMC11/DZ11-A, -B, -C, or -D async line multiplexer)",sdc_serial , 0 },
    /* 31 */ { "UEC","Univation RAINBOW-100 (or early DELQA PDP-11) CSMA/CD",sdc_lan , sls_csmacd },
    /* 32 */ { "KL8","KL8-J" , sdc_obsolete , 0 },
    /* 33 */ { "DS2","DSRVB DECserver 200 Lance CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 34 */ { "DMV","DMV11 interprocessor link" , sdc_other , 0 },
    /* 35 */ { "DS5","DSRVS DECserver 5xx CSMA/CD" , sdc_lan , sls_csmacd },
    /* 36 */ { "DPV","DPV11 sync line" , sdc_serial , 0 },
    /* 37 */ { "LQA","DELQA Q-bus Lance CSMA/CD" , sdc_lan , sls_csmacd },
    /* 38 */ { "DMF","DMF32 Unibus sync line" , sdc_serial , 0 },
    /* 39 */ { "SVA","DESVA VAXstation/uVAX integral CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 40 */ { "DMR","DMR11-AA/-AB/-AC/-AE Unibus interprocessor link" ,sdc_serial , 0 },
    /* 41 */ { "MUX","MUXserver 100 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 42 */ { "KMY","KMS11-PX Unibus sync line with X.25 Level 2 microcode", sdc_serial , 0 },
    /* 43 */ { "DEP","DEPCA PCSG/IBM-PC CSMA/CD" , sdc_lan , sls_csmacd },
    /* 44 */ { "KMX","KMS11-BD/BE Unibus sync line with X.25 Level 2 microcode"    , sdc_serial , 0 },
    /* 45 */ { "LTM","LTM (911) Ethernet monitor software" , sdc_lan ,sls_csmacd },
    /* 46 */ { "DMB","DMB-32 BI sync line multiplexer" , sdc_serial , 0 },
    /* 47 */ { "DES","DESNC CSMA/CD Encryption Module" , sdc_lan , sls_csmacd},
    /* 48 */ { "KCP","KCP Professional 3xx sync/async line" , sdc_serial , 0},
    /* 49 */ { "MX3","DSRZC MUXserver 300 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 50 */ { "SYN","DEMSA MicroServer sync line" , sdc_serial , 0 },
    /* 51 */ { "MEB","DEMEB multiport bridge CSMA/CD" , sdc_lan , sls_csmacd},
    /* 52 */ { "DSB","DSB32 BI sync line" , sdc_serial , 0 },
    /* 53 */ { "BAM","DEBAM LANbridge 200 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 54 */ { "DST","DST32 TEAMmate sync line (DEC423)" , sdc_serial , 0 },
    /* 55 */ { "FAT","DEFAT DataKit Server CSMA/CD" , sdc_lan , sls_csmacd },
    /* 56 */ { "RSM","DERSM Remote Segment Monitor " , sdc_other , 0 },
    /* 57 */ { "RES","DERES Remote Environmental Sensor " , sdc_other , 0 },
    /* 58 */ { "3C2","3Com 3C503 Etherlink II" , sdc_lan , sls_csmacd },
    /* 59 */ { "3CM","3Com 3C523 Etherlink/MC" , sdc_lan , sls_csmacd },
    /* 60 */ { "DS3","DSRVF DECServer 300 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 61 */ { "MF2","KA640 uVAX 3300 integral CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 62 */ { "MMR","DEMMR Ethernet Multiport Manageable Repeater " ,sdc_lan , sls_csmacd },
    /* 63 */ { "VIT","DETLB-N Vitalink TransLAN III/IV Bridge " , sdc_lan ,sls_csmacd },
    /* 64 */ { "VT5","DETLB-P/-T Vitalink TransLAN 350 Bridge, TransPATH 350 Brouter " , sdc_lan , sls_csmacd },
    /* 65 */ { "BNI","DEBNI LANcontroller 200 BI CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 66 */ { "MNA","DEMNA LANcontroller 400 XMI CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 67 */ { "PMX","PM2xx DECstation 3100 integral CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 68 */ { "NI5","NI5210-8 Interlan CSMA/CD for IBM PC XT/AT " , sdc_lan, sls_csmacd },
    /* 69 */ { "NI9","NI9210 Interlan CSMA/CD for IBM PS/2 " , sdc_lan ,sls_csmacd },
    /* 70 */ { "KMK","KMS11-K DataKit Unibus adapter" , sdc_other , 0 },
    /* 71 */ { "3CP","3Com 3C505 Etherlink Plus" , sdc_lan , sls_csmacd },
    /* 72 */ { "DP2","DSRVP DECserver 250 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 73 */ { "ISA","KA6xx VAX 4000 integral SGEC CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 74 */ { "DIV","DIV32 ISDNcontroller 100 Q-bus ISDN" , sdc_serial , 0},
    /* 75 */ { "QTA","DEQTA (DELQA-YM) Q-bus CSMA/CD" , sdc_lan , sls_csmacd},
    /* 76 */ { "B15","DEBET-AC LANbridge 150 CSMA/CD" , sdc_lan , sls_csmacd},
    /* 77 */ { "WD8","WD8003 Family CSMA/CD" , sdc_lan , sls_csmacd },
    /* 78 */ { "ILA","BICC ISOLAN 4110-2 PC/AT CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 79 */ { "ILM","BICC ISOLAN 4110-3 PC MicroChannel CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 80 */ { "APR","Apricot Xen-S and Qi CSMA/CD adapter" , sdc_lan ,sls_csmacd },
    /* 81 */ { "ASN","AST EtherNode CSMA/CD" , sdc_lan , sls_csmacd },
    /* 82 */ { "ASE","AST Ethernet CSMA/CD" , sdc_lan , sls_csmacd },
    /* 83 */ { "TRW","TRW HC-2001 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 84 */ { "EDX","Ethernet-XT/AT CSMA/CD" , sdc_lan , sls_csmacd },
    /* 85 */ { "EDA","Ethernet-AT CSMA/CD" , sdc_lan , sls_csmacd },
    /* 86 */ { "DR2","DSRVR WANrouter 250 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 87 */ { "SCC","DSRVR WANrouter 250 DUSCC serial link" , sdc_serial , 0},
    /* 88 */ { "DCA","DCA Series 300 Net Processor CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 89 */ { "TIA","LANcard/E CSMA/CD controllers" , sdc_lan , sls_csmacd},
    /* 90 */ { "FBN","DEFEB DECbridge 5xx CSMA/CD card" , sdc_lan ,sls_csmacd },
    /* 91 */ { "FEB","DEFEB DECbridge 5xx/6xx FDDI card" , sdc_lan , sls_fddi},
    /* 92 */ { "FCN","DEFCN DECconcentrator 500 FDDI concentrator" , sdc_lan, sls_fddi },
    /* 93 */ { "MFA","DEMFA FDDIcontroller 400 XMI FDDI" , sdc_lan , sls_fddi},
    /* 94 */ { "MXE","PMAD Turbochannel CSMA/CD" , sdc_lan , sls_csmacd },
    /* 95 */ { "CED","Cabletron Ethernet Desktop Network Interface CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 96 */ { "C20","3Com CS/200 terminal server CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 97 */ { "CS1","3Com CS/1 terminal server CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 98 */ { "C2M","3Com CS/210, CS/2000, CS/2100 terminal servers CSMA/CD", sdc_lan , sls_csmacd },
    /* 99 */ { "ACA","ACA/32000 system CSMA/CD" , sdc_lan , sls_csmacd },
    /* 100 */ { "GSM","Gandalf StarMaster CSMA/CD" , sdc_lan , sls_csmacd },
    /* 101 */ { "DSF","DSF32 WANcontroller 620 VAXft sync controller" ,sdc_serial , 0 },
    /* 102 */ { "CS5","3Com CS/50 terminal server CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 103 */ { "XIR","XIRCOM PE10B2 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 104 */ { "KFE","KFE52 VAXft CSMA/CD plus DSSI" , sdc_lan , sls_csmacd},
    /* 105 */ { "RT3","rtVAX-300 integral SGEC CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 106 */ { "SPI","Spiderport M250 Spiderport terminal server CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 107 */ { "FOR","LAT gateway CSMA/CD" , sdc_lan , sls_csmacd },
    /* 108 */ { "MER","Meridian CSMA/CD drivers" , sdc_lan , sls_csmacd },
    /* 109 */ { "PER","Persoft CSMA/CD drivers" , sdc_lan , sls_csmacd },
    /* 110 */ { "STR","AT&T StarLan-10 twisted pair comm link" , sdc_lan , sls_unknown },
    /* 111 */ { "MPS","KN220? DECstation 5400 SGEC integral CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 112 */ { "L20","LPS20-DA print server CSMA/CD" , sdc_lan , sls_csmacd},
    /* 113 */ { "VT2","DETLB-R Vitalink TransLAN 320 Bridge" , sdc_lan ,sls_csmacd },
    /* 114 */ { "DWT","VX100 VT1000 X terminal CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 115 */ { "WGB","DEWGB DECbridge 90 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 116 */ { "ZEN","Zenith Z-LAN4000 XT, AT and MicroChannel Z-LAN link" ,sdc_lan , sls_zlan },
    /* 117 */ { "TSS","Thursby S/W Systems CSMA/CD drivers" , sdc_lan ,sls_csmacd },
    /* 118 */ { "MNE","KN02-BA DECstation 5000 family integral CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 119 */ { "FZA","DEFZA TurboChannel FDDI" , sdc_lan , sls_fddi },
    /* 120 */ { "90L","DSRVD DECserver 90L terminal server CSMA/CD" , sdc_lan, sls_csmacd },
    /* 121 */ { "CIS","Cisco Systems CSMA/CD" , sdc_lan , sls_csmacd },
    /* 122 */ { "STC","STRTC terminal servers" , sdc_lan , sls_csmacd },
    /* 123 */ { "UBE","Ungermann-Bass PC2030, PC3030 CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 124 */ { "DW2","VXT terminal CSMA/CD" , sdc_lan , sls_csmacd },
    /* 125 */ { "FUE","Fujitsu Etherstar MB86950 CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 126 */ { "M38","DSRZD MUXServer 380 remote term. server CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 127 */ { "NTI","NTI Group PC Ethernet Card CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 128 */ { "LT2","LPS20-GA turbo print server CSMA/CD", sdc_lan ,sls_csmacd },
    /* 129 */ { "L17","LPS17 print server CSMA/CD", sdc_lan , sls_csmacd },
    /* 130 */ { "RAD","RADLINX LAN Gateway CSMA/CD" , sdc_lan , sls_csmacd },
    /* 131 */ { "INF","Infotron Commix series terminal server CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 132 */ { "XMX","Xyplex MAXserver series terminal server CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 133 */ { "NDI","NDIS driver, MS/DOS systems CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 134 */ { "ND2","NDIS driver, OS/2 systems CSMA/CD" , sdc_lan , sls_csmacd },
    /* 135 */ { "TRN","DEQRA LANcontroller 520 802.5 Token Ring" , sdc_lan ,sls_token },
    /* 136 */ { "DEV","Develcon Electronics Ltd. LAT gateway CSMA/CD" ,sdc_lan , sls_csmacd },
    /* 137 */ { "ACE","Acer 5220, 5270 CSMA/CD" , sdc_lan , sls_csmacd },
    /* 138 */ { "PNT","ProNet-4/18 #1390 802.5 Token Ring" , sdc_lan ,sls_token },
    /* 139 */ { "ISE","DNSAE DECnis 600 CSMA/CD line card" , sdc_lan ,sls_csmacd },
    /* 140 */ { "IST","DNSCB? DECnis 600 T1 sync line card" , sdc_serial , 0},
    /* 141 */ { "ISH","DNSCA? DECnis 600 64 kb HDLC line card" , sdc_serial ,0 },
    /* 142 */ { "ISF","DNSAF DECnis 600 FDDI line card" , sdc_lan , sls_fddi},
    /* 143 */ { "DR1","DSRVJ DECrouter 150 CSMA/CD", sdc_lan , sls_csmacd },
    /* 144 */ { "SC1","DSRVJ DECrouter 150 DUSCC serial", sdc_serial , 0 },
    /* 145 */ { "FB3","DEFEB-N DECbridge 6xx 3-port CSMA/CD" , sdc_lan ,sls_csmacd },
    /* 146 */ { 0 },
    /* 147 */ { 0 },
    /* 148 */ { 0 },
    /* 149 */ { "DSW","DSW21 single line sync" , sdc_serial , 0 },
    /* 150 */ { "DW4","DSW41/42 single/dual line sync" , sdc_serial , 0 },
    /* 151 */ { 0 },
    /* 152 */ { 0 },
    /* 153 */ { 0 },
    /* 154 */ { "ITC","DEC 4000 integral TGEC CSMA/CD", sdc_lan , sls_csmacd},
    /* 155 */ { 0 },
    /* 156 */ { "ACS","DSRVW DECserver 700 CSMA/CD", sdc_lan , sls_csmacd },
    /* 157 */ { "9LP","DSRVG DECserver 90L+ or LATprint", sdc_lan ,sls_csmacd },
    /* 158 */ { "92M","DSRVE DECserver 90TL CSMA/CD", sdc_lan , sls_csmacd },
    /* 159 */ { 0 },
    /* 160 */ { "FTA","DEFTA Turbochannel CSMA/CD", sdc_lan , sls_csmacd },
    /* 161 */ { "FAA","DEFAA Futurebus FDDI", sdc_lan , sls_fddi },
    /* 162 */ { "FEA","DEFEA EISA bus FDDI", sdc_lan , sls_fddi },
    /* 163 */ { 0 },
    /* 164 */ { 0 },
    /* 165 */ { "NMA","DENMA Network Management Agent", sdc_lan , sls_csmacd},
    /* 166 */ { "M32","DSRZE MUXserver 320 CSMA/CD", sdc_lan , sls_csmacd },
    /* 167 */ { "90W","DEWAR WANrouter-90 CSMA/CD", sdc_lan , sls_csmacd },
    /* 168 */ { "9WS","DEWAR WANrouter-90 sync", sdc_serial , 0 },
    /* 169 */ { "FQA","DEFQA Turbochannel Q-bus FDDI", sdc_lan, sls_fddi },
    /* 170 */ { "A35","DEC 3000 M400/500/900 CSMA/CD", sdc_lan , sls_csmacd},
    /* 171 */ { 0 },
    /* 172 */ { "V49","VAXstation 4000 Model 90 CSMA/CD", sdc_lan ,sls_csmacd },
    /* 173 */ { 0 },
    /* 174 */ { 0 },
    /* 175 */ { "TRE","DETRA-AA Turbochannel 802.5 token ring", sdc_lan ,sls_token },
    /* 176 */ { 0 },
    /* 177 */ { 0 },
    /* 178 */ { "XXX","DSRVH DECserver 90M",sdc_lan,sls_csmacd },
    /* 179 */ { 0 },
    /* 180 */ { 0, "DSRVZ DECserver 900TM", sdc_lan , sls_csmacd },
    /* 181 */ { "FGL", "DEFGL Gigaswitch FDDI", sdc_lan , sls_fddi },
    /* 182 */ { "ERA", "DE422 EISA CSMA/CD", sdc_lan , sls_csmacd },
    /* 183 */ { 0 },
    /* 184 */ { 0 },
    /* 185 */ { 0 },
    /* 186 */ { 0 },
    /* 187 */ { 0 },
    /* 188 */ { 0 },
    /* 189 */ { "TRE", "DW300 Proteon EISA Token Ring", sdc_lan , sls_token },
    /* 190 */ { 0 },
    /* 191 */ { 0 },
    /* 192 */ { 0 },
    /* 193 */ { 0 },
    /* 194 */ { 0 },
    /* 195 */ { 0, "DEFBA DECswitch 900 CSMA/CD", sdc_lan , sls_csmacd },
    /* 196 */ { 0, "DEFBA DECswitch 900 FDDI", sdc_lan , sls_fddi },
    /* 197 */ { 0 },
    /* 198 */ { 0 },
    /* 199 */ { 0 },
    /* 200 */ { 0 },
    /* 201 */ { 0, "DETMM DECrepeater 900TM CSMA/CD", sdc_lan, sls_csmacd },
    /* 202 */ { "ETA","DE425 EISA CSMA/CD", sdc_lan , sls_csmacd },
    /* 203 */ { "EWA","DE435 PCI CSMA/CD", sdc_lan , sls_csmacd },
    /* 204 */ { "FWA","DEFPZ(?) PCI FDDI", sdc_lan , sls_fddi },
    /* 205 */ { 0 },
    /* 206 */ { 0, "DECrepeater 900 Series CSMA/CD", sdc_lan , sls_csmacd },
    /* 207 */ { 0 },
    /* 208 */ { 0, "DEF6X DECconcentrator 900MX FDDI", sdc_lan , sls_fddi },
    /* 209 */ { 0 },
    /* 210 */ { 0 },
    /* 211 */ { 0 },
    /* 212 */ { 0 },
    /* 213 */ { "AZA", "DGLTA ATMworks 750 Turbochannel ATM", sdc_lan ,sls_unknown},
    /* 214 */ { 0 },
    /* 215 */ { 0 },
    /* 216 */ { "FPA", "DEFPA PCI FDDI", sdc_lan , sls_fddi },
    /* 217 */ { 0 },
    /* 218 */ { 0, "DESBF PEswitch 900TX FDDI", sdc_lan, sls_fddi },
    /* 219 */ { 0, "DESBF PEswitch 900TX CSMA/CD", sdc_lan, sls_csmacd },
    /* 220 */ { 0 },
    /* 221 */ { 0 },
    /* 222 */ { 0 },
    /* 223 */ { 0 },
    /* 224 */ { 0 },
    /* 225 */ { 0 },
    /* 226 */ { 0 },
    /* 227 */ { 0 },
    /* 228 */ { 0 },
    /* 229 */ { 0 },
    /* 230 */ { 0 },
    /* 231 */ { 0 },
    /* 232 */ { 0 },
    /* 233 */ { 0 },
    /* 234 */ { 0 },
    /* 235 */ { 0 },
    /* 236 */ { 0 },
    /* 237 */ { 0 },
    /* 238 */ { 0 },
    /* 229 */ { 0 },
    /* 240 */ { 0 },
    /* 241 */ { 0 },
    /* 242 */ { 0 },
    /* 243 */ { 0 },
    /* 244 */ { 0 },
    /* 245 */ { 0 },
    /* 246 */ { 0 },
    /* 247 */ { 0 },
    /* 248 */ { 0 },
    /* 249 */ { 0 },
    /* 250 */ { 0 },
    /* 251 */ { 0 },
    /* 252 */ { 0 },
    /* 253 */ { "ANA","DEANA Futurebus CSMA/CD", sdc_lan , sls_csmacd },
    /* 254 */ { 0 },
    /* 255 */ { 0 }
};

	fprintf(fpOut,"  Communication Device type (%d) ",id);

	if ((id > 0) 
        &&  (id < sizeof(sofdtab)/sizeof(sofdtab[0]))
        &&  (sofdtab[id].ste_mop_name != NULL))
	{
		fprintf(fpOut,"%s - %s",sofdtab[id].ste_mop_name,
                        sofdtab[id].ste_desc);
	}
	fputc('\n',fpOut);
}

static void  
display_HW_model(FILE *fpOut,unsigned char *info,unsigned char info_len)
{
        int i;
        unsigned long hwmodel;
        char *vaxes[499] = {
"Unknown model 0",
"VAX-11/780",
"VAX-11/782",
"VAX-11/750",
"VAX-11/730",
"VAX-11/785",
"MicroVAX I",
"VAXstation I",
"MicroVAX II",
"VAXstation II",
"VAXstation II/GPX",
"VAX 8600",
"VAX 8650",
"VAX 8200",
"VAX 8300",
"VAX 8530",
"VAX 8550",
"VAX 8700",
"VAX 8800",
"VAXstation 2000",
"MicroVAX 2000",
"VAXstation 2000/GPX",
"Processor 009",
"VAX 8250",
"VAX 8350",
"MicroVAX 3600 Series",
"VAXstation 3600 Series",
"VAXstation 3600 Series",
"VAX 6000-210",
"MicroVAX 3520",
"VAXstation 3520",
"VAX 8840",
"VAX 9RR",
"VAXserver II",
"Unknown MicroVAX II",
"MicroVAX II",
"Unknown VAXstation II",
"MicroVAX II",
"Unknown VAXstation II",
"VAXserver 2000",
"Unknown MicroVAX 2000",
"MicroVAX 2000",
"Unknown VAXstation 2000",
"MicroVAX 2000",
"Unknown VAXstation 2000",
"VAXserver 3600 Series",
"Unknown MicroVAX 3600 Series",
"MicroVAX 3600 Series",
"Unknown VAXstation 3600 Series",
"MicroVAX 3600 Series",
"Unknown VAXstation 3600 Series",
"VAXserver 3520",
"Unknown MicroVAX 3520",
"MicroVAX 3520",
"Unknown VAXstation 3520",
"VAXstation 8250L",
"Unknown VAXstation 8250L",
"MicroVAX 3100",
"VAXstation 3100",
"VAXstation 3100/GPX",
"VAXserver 3100",
"Unknown MicroVAX 3100",
"MicroVAX 3100",
"Unknown VAXstation 3100",
"MicroVAX 3100/GPX",
"Unknown VAXstation 3100",
"VAX 8500",
"VAX 8370",
"VAX 8650-PLUS",
"VAX 6000-220",
"VAX 6000-230",
"VAX 6000-240",
"VAX 6000-250",
"VAX 6000-260",
"VAX 6000-270",
"VAX 6000-280",
"VAX 6000-310",
"VAX 6000-320",
"VAX 6000-330",
"VAX 6000-340",
"VAX 6000-350",
"VAX 6000-360",
"VAX 6000-370",
"VAX 6000-380",
"VAX 8810",
"VAX 8820",
"VAX 8830",
"MicroVAX 3400 Series",
"VAXstation 3400 Series",
"VAXstation 3400 Series",
"VAXserver 3400 Series",
"Unknown MicroVAX 3400 Series",
"MicroVAX 3400 Series",
"Unknown VAXstation 3400 Series",
"MicroVAX 3400 Series",
"Unknown VAXstation 3400 Series",
"MicroVAX 2000",
"VAXstation 2000",
"VAXstation 2000/GPX",
"VAXstation 2000/GPX",
"VAXserver 6000-210",
"VAXserver 6000-220",
"VAXserver 6000-230",
"VAXserver 6000-240",
"VAXserver 6000-250",
"VAXserver 6000-260",
"VAXserver 6000-270",
"VAXserver 6000-280",
"VAXserver 6000-310",
"VAXserver 6000-320",
"VAXserver 6000-330",
"VAXserver 6000-340",
"VAXserver 6000-350",
"VAXserver 6000-360",
"VAXserver 6000-370",
"VAXserver 6000-380",
"Unknown VAX 6000-200 Series",
"Unknown VAX 6000-300 Series",
"MicroVAX 3900 Series",
"VAXserver 3900 Series",
"VAXstation 3900 Series",
"MicroVAX 3900 Series",
"Unknown MicroVAX 3900 Series",
"Unknown VAXstation 3900 Series",
"Industrial MicroVAX 2000",
"Industrial VAXserver 2000",
"Industrial VAXstation 2000",
"Industrial VAXstation 2000/GPX",
"Industrial MicroVAX 2000",
"Industrial MicroVAX 2000",
"Unknown MicroVAX 2000",
"Unknown VAXstation 2000",
"Unknown VAXstation 2000",
"MicroVAX 3540",
"VAXserver 3540",
"VAXstation 3540",
"MicroVAX 3540",
"MicroVAX 3560",
"VAXserver 3560",
"VAXstation 3560",
"MicroVAX 3560",
"MicroVAX 3580",
"VAXserver 3580",
"VAXstation 3580",
"MicroVAX 3580",
"MicroVAX 35A0",
"VAXserver 35A0",
"MicroVAX 3100",
"VAXstation 3100",
"VAXstation 3100/GPX",
"VAXserver 3100",
"Unknown MicroVAX 3100",
"MicroVAX 3100",
"Unknown VAXstation 3100",
"MicroVAX 3100/GPX",
"Unknown VAXstation 3100",
"MicroVAX 3100",
"VAXserver 3100",
"Unknown MicroVAX 3100",
"VAX 6000-410",
"VAX 6000-420",
"VAX 6000-430",
"VAX 6000-440",
"VAX 6000-450",
"VAX 6000-460",
"VAX 6000-470",
"VAX 6000-480",
"VAXserver 6000-410",
"VAXserver 6000-420",
"VAXserver 6000-430",
"VAXserver 6000-440",
"VAXserver 6000-450",
"VAXserver 6000-460",
"VAXserver 6000-470",
"VAXserver 6000-480",
"Unknown VAX 6000-400 Series",
"VAX 9000-210",
"VAX 9000-220",
"VAX 9000-410",
"VAX 9000-420",
"VAX 9000-430",
"VAX 9000-440",
"VAX 6305E",
"VAXserver 6305E",
"Unknown VAX 6305E Series",
"VAX 6000-510",
"VAX 6000-520",
"VAX 6000-530",
"VAX 6000-540",
"VAX 6000-550",
"VAX 6000-560",
"VAX 6000-570",
"VAX 6000-580",
"VAXserver 6000-510",
"VAXserver 6000-520",
"VAXserver 6000-530",
"VAXserver 6000-540",
"VAXserver 6000-550",
"VAXserver 6000-560",
"VAXserver 6000-570",
"VAXserver 6000-580",
"Unknown VAX 6000-500 Series",
"VAXstation 4000-30",
"MicroVAX 3100-30/40",
"Unknown VAXstation 4000-30",
"VAXstation 4000-30",
"MicroVAX 3100-30/40",
"Unknown VAXstation 4000-30",
"VAXserver 4000-30",
"MicroVAX 3100-30/40",
"Unknown VAXserver 4000-30",
"VAX 4000-300",
"VAX 4000-300",
"Unknown VAX 4000-300",
"VAXft-310",
"VAXstation 3100-M76",
"MicroVAX 3100-M76",
"Unknown VAXstation 3100-M76",
"VAXstation 3100-M76/SPX",
"MicroVAX 3100-M76/SPX",
"Unknown VAXstation 3100-M76",
"VAXserver 3100-M76",
"MicroVAX 3100-M76",
"Unknown VAXserver 3100-M76",
"VAXstation 3100-M76/GPX",
"MicroVAX 3100-M76/GPX",
"Unknown VAXstation 3100-M76",
"VAXstation 3100/SPX",
"MicroVAX 3100/SPX",
"Unknown VAXstation 3100",
"VAXstation 3100/SPX",
"MicroVAX 3100/SPX",
"Unknown VAXstation 3100",
"VAXstation 4000-30",
"MicroVAX 3100-30/40",
"Unknown VAXstation 4000-30",
"VAXstation 4000-60",
"MicroVAX 3100-80",
"Unknown VAXstation 4000-60",
"VAXstation 4000-60",
"MicroVAX 3100-80",
"Unknown VAXstation 4000-60",
"VAXstation 4000-60",
"MicroVAX 3100-80",
"Unknown VAXstation 4000-60",
"VAXserver 4000-60",
"MicroVAX 3100-80",
"Unknown VAXserver 4000-60",
"VAX 4000-200",
"VAX 4000-200",
"Unknown VAX 4000-200",
"MicroVAX 3820",
"VAXstation 3820",
"VAXserver 3820",
"Unknown MicroVAX 3820",
"MicroVAX 3820",
"Unknown VAXstation 3820",
"MicroVAX 3840",
"VAXserver 3840",
"VAXstation 3840",
"MicroVAX 3840",
"MicroVAX 3860",
"VAXserver 3860",
"VAXstation 3860",
"MicroVAX 3860",
"MicroVAX 3880",
"VAXserver 3880",
"VAXstation 3880",
"MicroVAX 3880",
"MicroVAX 38A0",
"VAXserver 38A0",
"Industrial MicroVAX 3100",
"Industrial VAXstation 3100",
"Industrial VAXstation 3100/GPX",
"Industrial VAXserver 3100",
"unknown MicroVAX 3100",
"Industrial MicroVAX 3100",
"Unknown MicroVAX 3100",
"Unknown MicroVAX 3100",
"Unknown MicroVAX 3100",
"Industrial VAXstation 3100/SPX",
"Industrial MicroVAX 3100",
"Unknown MicroVAX 3100",
"VAX 4000-600",
"VAX 4000-600",
"Unknown VAX 4000-600",
"VAX 4000-500",
"VAX 4000-500",
"Unknown VAX 4000-500",
"VAXft-310",
"VAXft-110",
"VAXft-110",
"VAXft-210",
"VAXft-210",
"VAX 6000-610",
"VAX 6000-620",
"VAX 6000-630",
"VAX 6000-640",
"VAX 6000-650",
"VAX 6000-660",
"VAX 6000-670",
"VAX 6000-680",
"VAXserver 6000-610",
"VAXserver 6000-620",
"VAXserver 6000-630",
"VAXserver 6000-640",
"VAXserver 6000-650",
"VAXserver 6000-660",
"VAXserver 6000-670",
"VAXserver 6000-680",
"Unknown VAX 6000-600 Series",
"VAX 9000-415",
"VAX 9000-425",
"VAX 9000-435",
"VAX 9000-445",
"VAX 9000-215",
"VAX 9000-225",
"VAX 9000-810",
"VAX 9000-820",
"VAX 9000-830",
"VAX 9000-840",
"VAX 9000-610",
"VAX 9000-620",
"VAXft-410",
"VAXft-410",
"VAXft-510",
"VAXft-510",
"VAXft-610",
"VAXft-610",
"VAXft-612",
"VAXft-612",
"VAXft-710",
"VAXft-710",
"VAXft-712",
"VAXft-712",
"MicroVAX 4000-VLC",
"VAXserver 4000-VLC",
"Unknown MicroVAX 4000-VLC",
"VAXstation 4000-VLC",
"VAXstation 4000-VLC",
"Unknown VAXstation 4000-VLC",
"VAX 9000-110",
"VAX 9000-120",
"VAX 9000-310",
"VAX 9000-320",
"VAX 9000-330",
"VAX 9000-340",
"VAX 9000-115",
"VAX 9000-125",
"VAX 9000-315",
"VAX 9000-325",
"VAX 9000-335",
"VAX 9000-345",
"VAX 9000-510",
"VAX 9000-520",
"VAX 9000-710",
"VAX 9000-720",
"VAX 9000-730",
"VAX 9000-740",
"VAX 7000-610",
"VAX 7000-620",
"VAX 7000-630",
"VAX 7000-640",
"VAX 7000-650",
"VAX 7000-660",
"VAX 7000-670",
"VAX 7000-680",
"VAXserver 7000-610",
"VAXserver 7000-620",
"VAXserver 7000-630",
"VAXserver 7000-640",
"VAXserver 7000-650",
"VAXserver 7000-660",
"VAXserver 7000-670",
"VAXserver 7000-680",
"Unknown VAX 7000-600 Series",
"VAXstation 4000-90",
"VAXstation 4000-90",
"Unknown VAXstation 4000-90",
"VAXstation 4000-90",
"VAXstation 4000-90",
"Unknown VAXstation 4000-90",
"MicroVAX 3100-90",
"MicroVAX 3100-90",
"Unknown MicroVAX 3100-90",
"MicroVAX 3100-95",
"MicroVAX 3100-95",
"Unknown MicroVAX 3100-95",
"VAX 4000-100",
"VAX 4000-100",
"Unknown VAX 4000-100",
"VAX 4000-105A",
"VAX 4000-105A",
"Unknown VAX 4000-105A",
"VAX 4000-400",
"VAX 4000-400",
"Unknown VAX 4000-400",
"VAX 10000-610",
"VAX 10000-620",
"VAX 10000-630",
"VAX 10000-640",
"VAX 10000-650",
"VAX 10000-660",
"VAX 10000-670",
"VAX 10000-680",
"VAXserver 10000-610",
"VAXserver 10000-620",
"VAXserver 10000-630",
"VAXserver 10000-640",
"VAXserver 10000-650",
"VAXserver 10000-660",
"VAXserver 10000-670",
"VAXserver 10000-680",
"Unknown VAX 10000-600 Series",
"PCVAX",
"PCVAX",
"VAX 7000-710",
"VAX 7000-720",
"VAX 7000-730",
"VAX 7000-740",
"VAX 7000-750",
"VAX 7000-760",
"VAX 7000-770",
"VAX 7000-780",
"VAXserver 7000-710",
"VAXserver 7000-720",
"VAXserver 7000-730",
"VAXserver 7000-740",
"VAXserver 7000-750",
"VAXserver 7000-760",
"VAXserver 7000-770",
"VAXserver 7000-780",
"Unknown VAX 7000-700 Series",
"VAX 10000-710",
"VAX 10000-720",
"VAX 10000-730",
"VAX 10000-740",
"VAX 10000-750",
"VAX 10000-760",
"VAX 10000-770",
"VAX 10000-780",
"VAXserver 10000-710",
"VAXserver 10000-720",
"VAXserver 10000-730",
"VAXserver 10000-740",
"VAXserver 10000-750",
"VAXserver 10000-760",
"VAXserver 10000-770",
"VAXserver 10000-780",
"Unknown VAX 10000-700 Series",
"VAX 4000-400A",
"VAX 4000-400A",
"Unknown VAX 4000-400A",
"VAX 4000-500A",
"VAX 4000-500A",
"Unknown VAX 4000-500A",
"VAX 4000-600A",
"VAX 4000-600A",
"Unknown VAX 4000-600A",
"VAX 4000-700A",
"VAX 4000-700A",
"Unknown VAX 4000-700A",
"VAX 4000-50",
"VAX 4000-50",
"Unknown VAX 4000-50",
"VAX 4000-55",
"VAX 4000-55",
"Unknown VAX 4000-55",
"VAX 4000-100A",
"VAX 4000-100A",
"Unknown VAX 4000-100A",
"VAXft-810",
"VAXft-810",
"VAXft-812",
"VAXft-812",
"VAXstation 4000-90A",
"VAXstation 4000-90A",
"Unknown VAXstation 4000-90A",
"VAXstation 4000-90A",
"VAXstation 4000-90A",
"Unknown VAXstation 4000-90A",
"VAXstation 4000-90B",
"VAXstation 4000-90B",
"Unknown VAXstation 4000-90B",
"VAXstation 4000-90B",
"VAXstation 4000-90B",
"Unknown VAXstation 4000-90B",
"VAX 4000-106A",
"VAX 4000-106A",
"Unknown VAX 4000-106A",
"MicroVAX 3100-85",
"MicroVAX 3100-85",
"Unknown MicroVAX 3100-85",
"MicroVAX 3100-96",
"MicroVAX 3100-96",
"Unknown MicroVAX 3100-96",
"VAX 4000-705A",
"VAX 4000-705A",
"Unknown VAX 4000-705A",
};
        struct 
        {
                unsigned long model;
                char *name;
        } alphas[] = {
1025,"DEC 7000 Model 610",
1026,"DEC 7000 Model 620",
1027,"DEC 7000 Model 630",
1028,"DEC 7000 Model 640",
1029,"DEC 10000 Model 610",
1030,"DEC 10000 Model 620",
1031,"DEC 10000 Model 630",
1032,"DEC 10000 Model 640",
1033,"DEC 3000 Model 500",
1034,"DEC 3000 Model 500S",
1035,"DEC 3000 Model 400W",
1036,"DEC 3000 Model 400S",
1037,"DEC 4000 Model 610",
1038,"DEC 4000 Model 620",
1039,"DEC 4000 Model 810",
1040,"DEC 4000 Model 820",
1041,"Appreciated, Debugged - Unsupported",
1042,"DEC 3000 Model 300",
1043,"DEC 3000 Model 300L",
1044,"DEC 2000 Model 300",
1045,"DEC 2000 Model 300S",
1046,"DEC 7000 Model 650",
1047,"DEC 7000 Model 660",
1048,"DEC 10000 Model 650",
1049,"DEC 10000 Model 660",
1050,"DEC 4000 Model 630",
1051,"DEC 4000 Model 640",
1052,"DEC 4000 Model 650",
1053,"DEC 4000 Model 660",
1054,"DEC 2000 Model 200",
1055,"DEC 2000 Model 200S",
1056,"DEC 3000 Model 500X",
1064,"DEC 3000 Model 600",
1071,"DEC 3000 Model 900",
1086,"AlphaServer 400 4/166",
1087,"AlphaStation 200 4/166",
1090,"AlphaServer 1000 4/266",
1094,"AlphaServer 8200 5/300",
1100,"AlphaServer 8420 5/300",
1119,"AlphaServer 2100 5/250",
1152,"AlphaStation 400 4/233",
1161,"AlphaStation 600 5/266",
1260,"AlphaStation 255 4/232",
1268,"AlphaStation 500/333",
1299,"AlphaServer 8420 5/350",
1301,"AlphaServer 8440 5/350",
1303,"AlphaServer 8460 5/350",
1593,"AlphaServer 300 4/266",
};
        if (info_len == 4)
        {
                memcpy(&hwmodel,info,4);
                fprintf(fpOut,"  HW Model = %u ",hwmodel);
                if (hwmodel < 1025)
                {
                        /* a VAX - look in table        */
                        if (hwmodel < 499)
                                fputs(vaxes[hwmodel],fpOut);
                        else
                                fputs("Unknown VAX",fpOut);
                }
                else
                {
                        /* an alpha - look at table     */
                        for (i=0; i < (sizeof(alphas)/sizeof(alphas[0])); i++)
                        {
                                if (alphas[i].model == hwmodel)
                                        break;
                        }
                        if (i < (sizeof(alphas)/sizeof(alphas[0])))
                                fputs(alphas[i].name,fpOut);
                        else
                                fputs("Unknown Alpha",fpOut);
                }
	        fputc('\n',fpOut);
        }
        else
        {
                fprintf(fpOut,"  Software ID related type 204 len %d\n",info_len);
                display_bytes(fpOut,info,info_len);
        }
}

static void 
display_uptime(FILE *fpOut,unsigned char *info,unsigned short len)
{
	unsigned long uptime,days,hours,mins,seconds;

	/*The first few bytes of this is the system uptime in 100ths second*/
	if (len > sizeof(uptime))
	{
		memcpy(&uptime,info,sizeof(uptime));
		days = uptime/8640000;
		uptime -= days * 8640000;		
		hours = uptime/360000;
		uptime -= hours*360000;
		mins = uptime/6000;
		uptime -= mins*6000;
		seconds = uptime/100;
		fprintf(fpOut,"  System has been up for %3d %02d:%02d:%02d\n",
			days,hours,mins,seconds);
	}
}

int
get_sysid_name(unsigned char *msg, unsigned short len, char *name)
{
        int found = FALSE;
        unsigned int n;
        unsigned char *msgptr, *info;
        SYSID_INFO_HDR *infohdr;

        if ((msg == NULL) || (len == 0))
                return FALSE;

        n = len - sizeof(SYSID_HDR);    /* length of other info data    */
        msgptr = &msg[sizeof(SYSID_HDR)];/* addr of first info           */
        while ((n > 0) && !found)
        {
                infohdr = (SYSID_INFO_HDR *)msgptr;
		info = msgptr + sizeof(SYSID_INFO_HDR);
		if (infohdr->itype == 0)
			break;
		switch(infohdr->itype)
		{
		case 11:
                        found = TRUE;
			break;				
		case 12:	
                        found = TRUE;
			break;				
                case 105:
                        found = TRUE;
                        break;
                case 203:
                        found = TRUE;
                        break;
		}
                n -= sizeof(SYSID_INFO_HDR) + infohdr->ilen;
                msgptr += sizeof(SYSID_INFO_HDR) + infohdr->ilen;
        }
        if (found)
        {
                memcpy(name,info,infohdr->ilen);
                name[infohdr->ilen] = '\0';
        }
        return found;
}

int
get_hw_address(unsigned char *msg, unsigned short len, unsigned char *addr)
{
        int found = FALSE;
        unsigned int n;
        unsigned char *msgptr, *info;
        SYSID_INFO_HDR *infohdr;

        if ((msg == NULL) || (len == 0))
                return FALSE;

        n = len - sizeof(SYSID_HDR);    /* length of other info data    */
        msgptr = &msg[sizeof(SYSID_HDR)];/* addr of first info           */
        while (n > 0)
        {
                infohdr = (SYSID_INFO_HDR *)msgptr;
		info = msgptr + sizeof(SYSID_INFO_HDR);
		if (infohdr->itype == 0)
			break;
		if (infohdr->itype == 7)
		{
                        found = TRUE;
			break;				
		}
                n -= sizeof(SYSID_INFO_HDR) + infohdr->ilen;
                msgptr += sizeof(SYSID_INFO_HDR) + infohdr->ilen;
        }
        if (found)
        {
                memcpy(addr,info,infohdr->ilen);
        }
        return found;
}
