/*
        PURGE_LISTEN_TABLE      - delete data from table older than x days

 COPYRIGHT NOTICE

 This software is COPYRIGHT  1997,2005, Ian Miller. 

 Released under licence described in aaareadme.txt 

 DISCLAIMER

 This software is provided "AS IS". The author makes no representations or
 warranties with respect to the software and specifically disclaim any implied
 warranties of merchantability or fitness for any particular purpose.

History	
=======

15-Jul-1997 I.Miller  V1.0-000	Created 
*/

#define VERSION "V1.0-000"
#if __DECC
#pragma module REPORT_LISTEN_TABLE VERSION
#else
#module REPORT_LISTEN_TABLE VERSION 
#endif

#include <stdio.h>
#include <descrip.h>
#include <libdef.h>
#include <starlet.h>
#include <lib$routines.h>

#include "ethernet.h"
#include "listen_table.h"
#include "sysid.h"
#include "listen_subs.h"

int main(int argc, char *argv[])
{
        int n,i;
        char buf[256];
        unsigned long ccode;
        unsigned short dt[4],today[4],result[4];
        $DESCRIPTOR(buf_dsc,"");

        /* get the delta time   */
        if (argc > 1)
        {
                strcpy(buf,argv[1]);
        }
        else
        {
                printf("Enter delta time :");
                gets(buf);
        }
        buf_dsc.dsc$a_pointer = buf;
        buf_dsc.dsc$w_length = strlen(buf);
        ccode = sys$bintim(&buf_dsc,dt);
        if (!(ccode & 1))
                return ccode;
        ccode = sys$gettim(today);
        if (!(ccode & 1))
                return ccode;
        ccode = lib$sub_times(today,dt,dt);
        if (!(ccode & 1))
                return ccode;
        /* load up the node table       */
        read_table();

        /* loop through nodes           */
        for (n=0; n < MAX_NODES; n++)
        {
                if (nodes[n].inuse)
                {
                        /* check node timestamp */
                        ccode = lib$sub_times(nodes[n].time,dt,result);
                        if (ccode & 1)
                        {
                                /* scan the protocols   */
                                for (i=0; i < MAX_PROTOCOLS; i++)
                                {
                                        switch(nodes[n].protocols[i].ethernet_protocol.format)
                                        {
                                        default:
                                        case LT_PROTOCOL_UNUSED:
						ccode = LIB$_NORMAL;
                                                break;
                                        case LT_PROTOCOL_ETH:
                                                ccode = lib$sub_times(nodes[n].protocols[i].ethernet_protocol.time,
                                                        dt,result);
                                                break;
                                        case LT_PROTOCOL_802:
                                                ccode = lib$sub_times(nodes[n].protocols[i].iee802_protocol.time,
                                                        dt,result);
                                                break;
                                        case LT_PROTOCOL_802E:
                                                ccode = lib$sub_times(nodes[n].protocols[i].iee802e_protocol.time,
                                                        dt,result);
                                                break;
                                        }
                                        if (ccode == LIB$_NEGTIM)
                                        {
                                                printf("purging protocol entry %d type 0x%02x for node (%d) ",
                                                        i,nodes[n].protocols[i].ethernet_protocol.format,n);
                                                display_addr(stdout,nodes[n].addr);
                                                putchar('\n');
                                                nodes[n].protocols[i].ethernet_protocol.format = LT_PROTOCOL_UNUSED;
                                        }
                                }
                                /* scan the addresses   */
                                for (i=0; i < MAX_ADDRESSES; i++)
                                {
                                        if (nodes[n].addresses[i].type != LT_ADDRESS_UNUSED)
                                        {
                                                ccode = lib$sub_times(nodes[n].addresses[i].time,dt,result);
                                                if (ccode == LIB$_NEGTIM)
                                                {
                                                        printf("purging address entry %d type 0x%02x for node (%d) ",
                                                                i,nodes[n].addresses[i].type,n);
                                                        display_addr(stdout,nodes[n].addr);
                                                        putchar('\n');
                                                        nodes[n].addresses[i].type = LT_ADDRESS_UNUSED;
                                                }
                                        }
                                }
                                /* scan the names       */
                                for (i=0; i < MAX_NAMES; i++)
                                {
                                        if (nodes[n].names[i].type != LT_NAME_UNUSED)
                                        {
                                                ccode = lib$sub_times(nodes[n].names[i].time,dt,result);
                                                if (ccode == LIB$_NEGTIM)
                                                {
                                                        printf("purging name %d type 0x%02x for node (%d) ",
                                                                i,nodes[n].names[i].type,n);
                                                        display_addr(stdout,nodes[n].addr);
                                                        putchar('\n');
                                                        nodes[n].names[i].type = LT_NAME_UNUSED;
                                                }
                                        }
                                }
                                /* scan the services    */
                                for (i=0; i < MAX_SERVICES; i++)
                                {
                                        if (nodes[n].services[i].type != LT_SERVICE_UNUSED)
                                        {
                                                ccode = lib$sub_times(nodes[n].services[i].time,dt,result);
                                                if (ccode == LIB$_NEGTIM)
                                                {
                                                        printf("purging service entry %d type 0x%02x for node (%d) ",
                                                                i,nodes[n].services[i].type,n);
                                                        display_addr(stdout,nodes[n].addr);
                                                        putchar('\n');
                                                        nodes[n].services[i].type = LT_SERVICE_UNUSED;
                                                }
                                        }
                                }
                        }
                        else if (ccode == LIB$_NEGTIM)
                        {
                                /* node not seen recently so get rid if it    */
                                printf("purging node (%d) ",n);
                                display_addr(stdout,nodes[n].addr);
                                putchar('\n');
                                nodes[n].inuse = FALSE;
                        }
                }
        }
        /* write table */
	write_table();	
}
